use specs::prelude::*;
use tui::{backend::Backend, layout::*, style::*, widgets::*, Frame};

use crate::logging::TuiLogWidget;
use crate::model::{Location, Metadata, State, TagId};

pub fn draw<B: Backend>(state: &State, mut frame: Frame<B>) {
    // Calculate main vertical layout division
    let chunks = Layout::default()
        .direction(Direction::Vertical)
        .constraints(
            [
                Constraint::Min(10),
                Constraint::Length(10),
                Constraint::Length(1),
                Constraint::Length(1),
            ]
            .as_ref(),
        )
        .split(frame.size());

    {
        let storage_view = Layout::default()
            .direction(Direction::Horizontal)
            .constraints(
                [Constraint::Percentage(50), Constraint::Percentage(50)]
                    .as_ref(),
            )
            .split(chunks[0]);

        let (ids, metas, locs) = state.world.system_data::<(
            ReadStorage<TagId>,
            ReadStorage<Metadata>,
            ReadStorage<Location>,
        )>();

        {
            let items_in_storage = (&ids, &metas, &locs)
                .join()
                .filter(|(_, _, loc)| **loc == Location::InStorage)
                .map(|(id, meta, _)| {
                    Row::Data(vec![&id.0, &meta.name].into_iter())
                });

            let in_storage_table =
                Table::new(["Tag Id", "Name"].iter(), items_in_storage)
                    .block(
                        Block::default()
                            .title("Items in storage")
                            .borders(Borders::ALL),
                    )
                    .header_style(Style::default().modifier(Modifier::BOLD))
                    .widths(&[
                        Constraint::Length(15),
                        Constraint::Percentage(100),
                    ]);
            frame.render_widget(in_storage_table, storage_view[0]);
        }
        {
            let rented_items = (&ids, &metas, &locs)
                .join()
                .filter_map(|(id, meta, loc)| match loc {
                    Location::RentedTo { renter } => Some((id, meta, renter)),
                    _ => None,
                })
                .map(|(id, meta, renter)| {
                    Row::Data(vec![&id.0, &meta.name, &renter].into_iter())
                });

            let rented_items_table =
                Table::new(["Tag Id", "Name", "Renter"].iter(), rented_items)
                    .block(
                        Block::default()
                            .title("Rented items")
                            .borders(Borders::ALL),
                    )
                    .header_style(Style::default().modifier(Modifier::BOLD))
                    .widths(&[
                        Constraint::Length(15),
                        Constraint::Percentage(100),
                        Constraint::Length(15),
                    ]);
            frame.render_widget(rented_items_table, storage_view[1]);
        }
    }
    {
        let logs = TuiLogWidget {
            state: &state.log_state,
        };
        frame.render_widget(logs, chunks[1]);
    }
    {
        let text = [
            Text::styled(
                state.menu_status.prompt(),
                Style::default().modifier(Modifier::BOLD),
            ),
            Text::styled("> ", Style::default().modifier(Modifier::BOLD)),
            Text::raw(&state.input),
            Text::styled("_", Style::default().modifier(Modifier::SLOW_BLINK)),
        ];
        let input_widget = Paragraph::new(text.iter())
            .alignment(Alignment::Left)
            .wrap(false);
        frame.render_widget(input_widget, chunks[2]);
    }
    {
        let text = [Text::raw(concat!(
            "1 = [register new item]   ",
            "2 = [rent item(s)]   ",
            "3 = [return item(s)]   ",
            "q = [quit]   ",
            "esc = [back]",
        ))];
        let instructions_widget = Paragraph::new(text.iter())
            .alignment(Alignment::Left)
            .wrap(false);
        frame.render_widget(instructions_widget, chunks[3]);
    }
}
